@@ -22,8 +22,8 @@
 #include "parasail/internal_sse.h"
 
 #define SG_STATS
-#define SG_SUFFIX _scan_sse41_128_16
-#define SG_SUFFIX_PROF _scan_profile_sse41_128_16
+#define SG_SUFFIX _scan_sse41_128_8
+#define SG_SUFFIX_PROF _scan_profile_sse41_128_8
 #include "sg_helper.h"
 
 
@@ -37,14 +37,22 @@ static inline void arr_store_si128(
         int32_t d,
         int32_t dlen)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 1);
-    array[1LL*(2*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 2);
-    array[1LL*(3*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 3);
-    array[1LL*(4*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 4);
-    array[1LL*(5*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 5);
-    array[1LL*(6*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 6);
-    array[1LL*(7*seglen+t)*dlen + d] = (int16_t)_mm_extract_epi16(vH, 7);
+    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  0);
+    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  1);
+    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  2);
+    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  3);
+    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  4);
+    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  5);
+    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  6);
+    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  7);
+    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  8);
+    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH,  9);
+    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 10);
+    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 11);
+    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 12);
+    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 13);
+    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 14);
+    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)_mm_extract_epi8(vH, 15);
 }
 #endif
 
@@ -55,27 +63,35 @@ static inline void arr_store_col(
         int32_t t,
         int32_t seglen)
 {
-    col[0*seglen+t] = (int16_t)_mm_extract_epi16(vH, 0);
-    col[1*seglen+t] = (int16_t)_mm_extract_epi16(vH, 1);
-    col[2*seglen+t] = (int16_t)_mm_extract_epi16(vH, 2);
-    col[3*seglen+t] = (int16_t)_mm_extract_epi16(vH, 3);
-    col[4*seglen+t] = (int16_t)_mm_extract_epi16(vH, 4);
-    col[5*seglen+t] = (int16_t)_mm_extract_epi16(vH, 5);
-    col[6*seglen+t] = (int16_t)_mm_extract_epi16(vH, 6);
-    col[7*seglen+t] = (int16_t)_mm_extract_epi16(vH, 7);
+    col[ 0*seglen+t] = (int8_t)_mm_extract_epi8(vH,  0);
+    col[ 1*seglen+t] = (int8_t)_mm_extract_epi8(vH,  1);
+    col[ 2*seglen+t] = (int8_t)_mm_extract_epi8(vH,  2);
+    col[ 3*seglen+t] = (int8_t)_mm_extract_epi8(vH,  3);
+    col[ 4*seglen+t] = (int8_t)_mm_extract_epi8(vH,  4);
+    col[ 5*seglen+t] = (int8_t)_mm_extract_epi8(vH,  5);
+    col[ 6*seglen+t] = (int8_t)_mm_extract_epi8(vH,  6);
+    col[ 7*seglen+t] = (int8_t)_mm_extract_epi8(vH,  7);
+    col[ 8*seglen+t] = (int8_t)_mm_extract_epi8(vH,  8);
+    col[ 9*seglen+t] = (int8_t)_mm_extract_epi8(vH,  9);
+    col[10*seglen+t] = (int8_t)_mm_extract_epi8(vH, 10);
+    col[11*seglen+t] = (int8_t)_mm_extract_epi8(vH, 11);
+    col[12*seglen+t] = (int8_t)_mm_extract_epi8(vH, 12);
+    col[13*seglen+t] = (int8_t)_mm_extract_epi8(vH, 13);
+    col[14*seglen+t] = (int8_t)_mm_extract_epi8(vH, 14);
+    col[15*seglen+t] = (int8_t)_mm_extract_epi8(vH, 15);
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sg_flags_stats_table_scan_sse41_128_16
-#define PNAME parasail_sg_flags_stats_table_scan_profile_sse41_128_16
+#define FNAME parasail_sg_flags_stats_table_scan_sse41_128_8
+#define PNAME parasail_sg_flags_stats_table_scan_profile_sse41_128_8
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sg_flags_stats_rowcol_scan_sse41_128_16
-#define PNAME parasail_sg_flags_stats_rowcol_scan_profile_sse41_128_16
+#define FNAME parasail_sg_flags_stats_rowcol_scan_sse41_128_8
+#define PNAME parasail_sg_flags_stats_rowcol_scan_profile_sse41_128_8
 #else
-#define FNAME parasail_sg_flags_stats_scan_sse41_128_16
-#define PNAME parasail_sg_flags_stats_scan_profile_sse41_128_16
+#define FNAME parasail_sg_flags_stats_scan_sse41_128_8
+#define PNAME parasail_sg_flags_stats_scan_profile_sse41_128_8
 #endif
 #endif
 
@@ -85,7 +101,7 @@ parasail_result_t* FNAME(
         const int open, const int gap, const parasail_matrix_t *matrix,
         int s1_beg, int s1_end, int s2_beg, int s2_end)
 {
-    parasail_profile_t *profile = parasail_profile_create_stats_sse_128_16(s1, s1Len, matrix);
+    parasail_profile_t *profile = parasail_profile_create_stats_sse_128_8(s1, s1Len, matrix);
     parasail_result_t *result = PNAME(profile, s2, s2Len, open, gap, s1_beg, s1_end, s2_beg, s2_end);
     parasail_profile_free(profile);
     return result;
@@ -104,13 +120,13 @@ parasail_result_t* PNAME(
     int32_t end_query = s1Len-1;
     int32_t end_ref = s2Len-1;
     const parasail_matrix_t *matrix = profile->matrix;
-    const int32_t segWidth = 8; /* number of values in vector unit */
+    const int32_t segWidth = 16; /* number of values in vector unit */
     const int32_t segLen = (s1Len + segWidth - 1) / segWidth;
     const int32_t offset = (s1Len - 1) % segLen;
     const int32_t position = (segWidth - 1) - (s1Len - 1) / segLen;
-    __m128i* const restrict pvP  = (__m128i*)profile->profile16.score;
-    __m128i* const restrict pvPm = (__m128i*)profile->profile16.matches;
-    __m128i* const restrict pvPs = (__m128i*)profile->profile16.similar;
+    __m128i* const restrict pvP  = (__m128i*)profile->profile8.score;
+    __m128i* const restrict pvPm = (__m128i*)profile->profile8.matches;
+    __m128i* const restrict pvPs = (__m128i*)profile->profile8.similar;
     __m128i* const restrict pvE  = parasail_memalign___m128i(16, segLen);
     __m128i* const restrict pvEM = parasail_memalign___m128i(16, segLen);
     __m128i* const restrict pvES = parasail_memalign___m128i(16, segLen);
@@ -125,31 +141,31 @@ parasail_result_t* PNAME(
     __m128i* const restrict pvHLMax = parasail_memalign___m128i(16, segLen);
     __m128i* const restrict pvGapper = parasail_memalign___m128i(16, segLen);
     __m128i* const restrict pvGapperL = parasail_memalign___m128i(16, segLen);
-    int16_t* const restrict boundary = parasail_memalign_int16_t(16, s2Len+1);
-    __m128i vGapO = _mm_set1_epi16(open);
-    __m128i vGapE = _mm_set1_epi16(gap);
-    const int16_t NEG_LIMIT = (-open < matrix->min ?
-        INT16_MIN + open : INT16_MIN - matrix->min) + 1;
-    const int16_t POS_LIMIT = INT16_MAX - matrix->max - 1;
+    int8_t* const restrict boundary = parasail_memalign_int8_t(16, s2Len+1);
+    __m128i vGapO = _mm_set1_epi8(open);
+    __m128i vGapE = _mm_set1_epi8(gap);
+    const int8_t NEG_LIMIT = (-open < matrix->min ?
+        INT8_MIN + open : INT8_MIN - matrix->min) + 1;
+    const int8_t POS_LIMIT = INT8_MAX - matrix->max - 1;
     __m128i vZero = _mm_setzero_si128();
-    __m128i vOne = _mm_set1_epi16(1);
-    int16_t score = NEG_LIMIT;
-    int16_t matches = 0;
-    int16_t similar = 0;
-    int16_t length = 0;
-    __m128i vNegLimit = _mm_set1_epi16(NEG_LIMIT);
-    __m128i vPosLimit = _mm_set1_epi16(POS_LIMIT);
+    __m128i vOne = _mm_set1_epi8(1);
+    int8_t score = NEG_LIMIT;
+    int8_t matches = 0;
+    int8_t similar = 0;
+    int8_t length = 0;
+    __m128i vNegLimit = _mm_set1_epi8(NEG_LIMIT);
+    __m128i vPosLimit = _mm_set1_epi8(POS_LIMIT);
     __m128i vSaturationCheckMin = vPosLimit;
     __m128i vSaturationCheckMax = vNegLimit;
     __m128i vMaxH = vNegLimit;
     __m128i vMaxM = vNegLimit;
     __m128i vMaxS = vNegLimit;
     __m128i vMaxL = vNegLimit;
-    __m128i vPosMask = _mm_cmpeq_epi16(_mm_set1_epi16(position),
-            _mm_set_epi16(0,1,2,3,4,5,6,7));
+    __m128i vPosMask = _mm_cmpeq_epi8(_mm_set1_epi8(position),
+            _mm_set_epi8(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15));
     __m128i vNegInfFront = vZero;
     __m128i vSegLenXgap;
-    __m128i vSegLen = _mm_slli_si128(_mm_set1_epi16(segLen), 2);
+    __m128i vSegLen = _mm_slli_si128(_mm_set1_epi8(segLen), 1);
 #ifdef PARASAIL_TABLE
     parasail_result_t *result = parasail_result_new_table3(segLen*segWidth, s2Len);
 #else
@@ -160,9 +176,9 @@ parasail_result_t* PNAME(
 #endif
 #endif
 
-    vNegInfFront = _mm_insert_epi16(vNegInfFront, NEG_LIMIT, 0);
-    vSegLenXgap = _mm_add_epi16(vNegInfFront,
-            _mm_slli_si128(_mm_set1_epi16(-segLen*gap), 2));
+    vNegInfFront = _mm_insert_epi8(vNegInfFront, NEG_LIMIT, 0);
+    vSegLenXgap = _mm_adds_epi8(vNegInfFront,
+            _mm_slli_si128(_mm_set1_epi8(-segLen*gap), 1));
 
     parasail_memset___m128i(pvHM, vZero, segLen);
     parasail_memset___m128i(pvHS, vZero, segLen);
@@ -172,13 +188,13 @@ parasail_result_t* PNAME(
     parasail_memset___m128i(pvES, vZero, segLen);
     parasail_memset___m128i(pvEL, vZero, segLen);
     {
-        __m128i vGapper = _mm_sub_epi16(vZero,vGapO);
+        __m128i vGapper = _mm_subs_epi8(vZero,vGapO);
         __m128i vGapperL = vOne;
         for (i=segLen-1; i>=0; --i) {
             _mm_store_si128(pvGapper+i, vGapper);
             _mm_store_si128(pvGapperL+i, vGapperL);
-            vGapper = _mm_sub_epi16(vGapper, vGapE);
-            vGapperL = _mm_add_epi16(vGapperL, vOne);
+            vGapper = _mm_subs_epi8(vGapper, vGapE);
+            vGapperL = _mm_adds_epi8(vGapperL, vOne);
         }
     }
 
@@ -187,10 +203,10 @@ parasail_result_t* PNAME(
         int32_t index = 0;
         for (i=0; i<segLen; ++i) {
             int32_t segNum = 0;
-            __m128i_16_t h;
+            __m128i_8_t h;
             for (segNum=0; segNum<segWidth; ++segNum) {
                 int64_t tmp = s1_beg ? 0 : (-open-gap*(segNum*segLen+i));
-                h.v[segNum] = tmp < INT16_MIN ? INT16_MIN : tmp;
+                h.v[segNum] = tmp < INT8_MIN ? INT8_MIN : tmp;
             }
             _mm_store_si128(&pvH[index], h.m);
             ++index;
@@ -202,7 +218,7 @@ parasail_result_t* PNAME(
         boundary[0] = 0;
         for (i=1; i<=s2Len; ++i) {
             int64_t tmp = s2_beg ? 0 : (-open-gap*(i-1));
-            boundary[i] = tmp < INT16_MIN ? INT16_MIN : tmp;
+            boundary[i] = tmp < INT8_MIN ? INT8_MIN : tmp;
         }
     }
 
@@ -250,15 +266,15 @@ parasail_result_t* PNAME(
         vHpM = _mm_load_si128(pvHM+(segLen-1));
         vHpS = _mm_load_si128(pvHS+(segLen-1));
         vHpL = _mm_load_si128(pvHL+(segLen-1));
-        vHp = _mm_slli_si128(vHp, 2);
-        vHpM = _mm_slli_si128(vHpM, 2);
-        vHpS = _mm_slli_si128(vHpS, 2);
-        vHpL = _mm_slli_si128(vHpL, 2);
-        vHp = _mm_insert_epi16(vHp, boundary[j], 0);
+        vHp = _mm_slli_si128(vHp, 1);
+        vHpM = _mm_slli_si128(vHpM, 1);
+        vHpS = _mm_slli_si128(vHpS, 1);
+        vHpL = _mm_slli_si128(vHpL, 1);
+        vHp = _mm_insert_epi8(vHp, boundary[j], 0);
         pvW = pvP + matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWM= pvPm+ matrix->mapper[(unsigned char)s2[j]]*segLen;
         pvWS= pvPs+ matrix->mapper[(unsigned char)s2[j]]*segLen;
-        vHt = _mm_sub_epi16(vNegLimit, pvGapper[0]);
+        vHt = _mm_subs_epi8(vNegLimit, pvGapper[0]);
         vF = vNegLimit;
         vFM = vZero;
         vFS = vZero;
@@ -277,30 +293,30 @@ parasail_result_t* PNAME(
             vWS = _mm_load_si128(pvWS+i);
             vGapper = _mm_load_si128(pvGapper+i);
             vGapperL = _mm_load_si128(pvGapperL+i);
-            vE_opn = _mm_sub_epi16(vH, vGapO);
-            vE_ext = _mm_sub_epi16(vE, vGapE);
-            case1 = _mm_cmpgt_epi16(vE_opn, vE_ext);
-            vE = _mm_max_epi16(vE_opn, vE_ext);
+            vE_opn = _mm_subs_epi8(vH, vGapO);
+            vE_ext = _mm_subs_epi8(vE, vGapE);
+            case1 = _mm_cmpgt_epi8(vE_opn, vE_ext);
+            vE = _mm_max_epi8(vE_opn, vE_ext);
             vEM = _mm_blendv_epi8(vEM, vHM, case1);
             vES = _mm_blendv_epi8(vES, vHS, case1);
             vEL = _mm_blendv_epi8(vEL, vHL, case1);
-            vEL = _mm_add_epi16(vEL, vOne);
-            vGapper = _mm_add_epi16(vHt, vGapper);
+            vEL = _mm_adds_epi8(vEL, vOne);
+            vGapper = _mm_adds_epi8(vHt, vGapper);
             case1 = _mm_or_si128(
-                    _mm_cmpgt_epi16(vF, vGapper),
-                    _mm_cmpeq_epi16(vF, vGapper));
-            vF = _mm_max_epi16(vF, vGapper);
+                    _mm_cmpgt_epi8(vF, vGapper),
+                    _mm_cmpeq_epi8(vF, vGapper));
+            vF = _mm_max_epi8(vF, vGapper);
             vFM = _mm_blendv_epi8(vHtM, vFM, case1);
             vFS = _mm_blendv_epi8(vHtS, vFS, case1);
             vFL = _mm_blendv_epi8(
-                    _mm_add_epi16(vHtL, vGapperL),
+                    _mm_adds_epi8(vHtL, vGapperL),
                     vFL, case1);
-            vHp = _mm_add_epi16(vHp, vW);
-            vHpM = _mm_add_epi16(vHpM, vWM);
-            vHpS = _mm_add_epi16(vHpS, vWS);
-            vHpL = _mm_add_epi16(vHpL, vOne);
-            case1 = _mm_cmpgt_epi16(vE, vHp);
-            vHt = _mm_max_epi16(vE, vHp);
+            vHp = _mm_adds_epi8(vHp, vW);
+            vHpM = _mm_adds_epi8(vHpM, vWM);
+            vHpS = _mm_adds_epi8(vHpS, vWS);
+            vHpL = _mm_adds_epi8(vHpL, vOne);
+            case1 = _mm_cmpgt_epi8(vE, vHp);
+            vHt = _mm_max_epi8(vE, vHp);
             vHtM = _mm_blendv_epi8(vHpM, vEM, case1);
             vHtS = _mm_blendv_epi8(vHpS, vES, case1);
             vHtL = _mm_blendv_epi8(vHpL, vEL, case1);
@@ -319,50 +335,50 @@ parasail_result_t* PNAME(
         }
 
         /* pseudo prefix scan on F and H */
-        vHt = _mm_slli_si128(vHt, 2);
-        vHtM = _mm_slli_si128(vHtM, 2);
-        vHtS = _mm_slli_si128(vHtS, 2);
-        vHtL = _mm_slli_si128(vHtL, 2);
-        vHt = _mm_insert_epi16(vHt, boundary[j+1], 0);
+        vHt = _mm_slli_si128(vHt, 1);
+        vHtM = _mm_slli_si128(vHtM, 1);
+        vHtS = _mm_slli_si128(vHtS, 1);
+        vHtL = _mm_slli_si128(vHtL, 1);
+        vHt = _mm_insert_epi8(vHt, boundary[j+1], 0);
         vGapper = _mm_load_si128(pvGapper);
         vGapperL = _mm_load_si128(pvGapperL);
-        vGapper = _mm_add_epi16(vHt, vGapper);
+        vGapper = _mm_adds_epi8(vHt, vGapper);
         case1 = _mm_or_si128(
-                _mm_cmpgt_epi16(vGapper, vF),
-                _mm_cmpeq_epi16(vGapper, vF));
-        vF = _mm_max_epi16(vF, vGapper);
+                _mm_cmpgt_epi8(vGapper, vF),
+                _mm_cmpeq_epi8(vGapper, vF));
+        vF = _mm_max_epi8(vF, vGapper);
         vFM = _mm_blendv_epi8(vFM, vHtM, case1);
         vFS = _mm_blendv_epi8(vFS, vHtS, case1);
         vFL = _mm_blendv_epi8(
                 vFL,
-                _mm_add_epi16(vHtL, vGapperL),
+                _mm_adds_epi8(vHtL, vGapperL),
                 case1);
         for (i=0; i<segWidth-2; ++i) {
-            __m128i vFt = _mm_slli_si128(vF, 2);
-            __m128i vFtM = _mm_slli_si128(vFM, 2);
-            __m128i vFtS = _mm_slli_si128(vFS, 2);
-            __m128i vFtL = _mm_slli_si128(vFL, 2);
-            vFt = _mm_add_epi16(vFt, vSegLenXgap);
+            __m128i vFt = _mm_slli_si128(vF, 1);
+            __m128i vFtM = _mm_slli_si128(vFM, 1);
+            __m128i vFtS = _mm_slli_si128(vFS, 1);
+            __m128i vFtL = _mm_slli_si128(vFL, 1);
+            vFt = _mm_adds_epi8(vFt, vSegLenXgap);
             case1 = _mm_or_si128(
-                    _mm_cmpgt_epi16(vFt, vF),
-                    _mm_cmpeq_epi16(vFt, vF));
-            vF = _mm_max_epi16(vF, vFt);
+                    _mm_cmpgt_epi8(vFt, vF),
+                    _mm_cmpeq_epi8(vFt, vF));
+            vF = _mm_max_epi8(vF, vFt);
             vFM = _mm_blendv_epi8(vFM, vFtM, case1);
             vFS = _mm_blendv_epi8(vFS, vFtS, case1);
             vFL = _mm_blendv_epi8(
                     vFL,
-                    _mm_add_epi16(vFtL, vSegLen),
+                    _mm_adds_epi8(vFtL, vSegLen),
                     case1);
         }
 
         /* calculate final H */
-        vF = _mm_slli_si128(vF, 2);
-        vFM = _mm_slli_si128(vFM, 2);
-        vFS = _mm_slli_si128(vFS, 2);
-        vFL = _mm_slli_si128(vFL, 2);
-        vF = _mm_add_epi16(vF, vNegInfFront);
-        case1 = _mm_cmpgt_epi16(vF, vHt);
-        vH = _mm_max_epi16(vF, vHt);
+        vF = _mm_slli_si128(vF, 1);
+        vFM = _mm_slli_si128(vFM, 1);
+        vFS = _mm_slli_si128(vFS, 1);
+        vFL = _mm_slli_si128(vFL, 1);
+        vF = _mm_adds_epi8(vF, vNegInfFront);
+        case1 = _mm_cmpgt_epi8(vF, vHt);
+        vH = _mm_max_epi8(vF, vHt);
         vHM = _mm_blendv_epi8(vHtM, vFM, case1);
         vHS = _mm_blendv_epi8(vHtS, vFS, case1);
         vHL = _mm_blendv_epi8(vHtL, vFL, case1);
@@ -375,18 +391,18 @@ parasail_result_t* PNAME(
             vEM = _mm_load_si128(pvEM+i);
             vES = _mm_load_si128(pvES+i);
             vEL = _mm_load_si128(pvEL+i);
-            vF_opn = _mm_sub_epi16(vH, vGapO);
-            vF_ext = _mm_sub_epi16(vF, vGapE);
-            vF = _mm_max_epi16(vF_opn, vF_ext);
-            case1 = _mm_cmpgt_epi16(vF_opn, vF_ext);
+            vF_opn = _mm_subs_epi8(vH, vGapO);
+            vF_ext = _mm_subs_epi8(vF, vGapE);
+            vF = _mm_max_epi8(vF_opn, vF_ext);
+            case1 = _mm_cmpgt_epi8(vF_opn, vF_ext);
             vFM = _mm_blendv_epi8(vFM, vHM, case1);
             vFS = _mm_blendv_epi8(vFS, vHS, case1);
             vFL = _mm_blendv_epi8(vFL, vHL, case1);
-            vFL = _mm_add_epi16(vFL, vOne);
-            vH = _mm_max_epi16(vHp, vE);
-            vH = _mm_max_epi16(vH, vF);
-            case1 = _mm_cmpeq_epi16(vH, vHp);
-            case2 = _mm_cmpeq_epi16(vH, vF);
+            vFL = _mm_adds_epi8(vFL, vOne);
+            vH = _mm_max_epi8(vHp, vE);
+            vH = _mm_max_epi8(vH, vF);
+            case1 = _mm_cmpeq_epi8(vH, vHp);
+            case2 = _mm_cmpeq_epi8(vH, vF);
             vHM = _mm_blendv_epi8(
                     _mm_blendv_epi8(vEM, vFM, case2),
                     vHpM, case1);
@@ -400,11 +416,11 @@ parasail_result_t* PNAME(
             _mm_store_si128(pvHM+i, vHM);
             _mm_store_si128(pvHS+i, vHS);
             _mm_store_si128(pvHL+i, vHL);
-            vSaturationCheckMin = _mm_min_epi16(vSaturationCheckMin, vH);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vH);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = _mm_max_epi16(vSaturationCheckMax, vHL);
+            vSaturationCheckMin = _mm_min_epi8(vSaturationCheckMin, vH);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vH);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = _mm_max_epi8(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
             arr_store_si128(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
             arr_store_si128(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
@@ -419,8 +435,8 @@ parasail_result_t* PNAME(
             vHM = _mm_load_si128(pvHM + offset);
             vHS = _mm_load_si128(pvHS + offset);
             vHL = _mm_load_si128(pvHL + offset);
-            cond_max = _mm_cmpgt_epi16(vH, vMaxH);
-            vMaxH = _mm_max_epi16(vMaxH, vH);
+            cond_max = _mm_cmpgt_epi8(vH, vMaxH);
+            vMaxH = _mm_max_epi8(vMaxH, vH);
             vMaxM = _mm_blendv_epi8(vMaxM, vHM, cond_max);
             vMaxS = _mm_blendv_epi8(vMaxS, vHS, cond_max);
             vMaxL = _mm_blendv_epi8(vMaxL, vHL, cond_max);
@@ -429,15 +445,15 @@ parasail_result_t* PNAME(
             }
 #ifdef PARASAIL_ROWCOL
             for (k=0; k<position; ++k) {
-                vH = _mm_slli_si128(vH, 2);
-                vHM = _mm_slli_si128(vHM, 2);
-                vHS = _mm_slli_si128(vHS, 2);
-                vHL = _mm_slli_si128(vHL, 2);
+                vH = _mm_slli_si128(vH, 1);
+                vHM = _mm_slli_si128(vHM, 1);
+                vHS = _mm_slli_si128(vHS, 1);
+                vHL = _mm_slli_si128(vHL, 1);
             }
-            result->stats->rowcols->score_row[j] = (int16_t) _mm_extract_epi16 (vH, 7);
-            result->stats->rowcols->matches_row[j] = (int16_t) _mm_extract_epi16 (vHM, 7);
-            result->stats->rowcols->similar_row[j] = (int16_t) _mm_extract_epi16 (vHS, 7);
-            result->stats->rowcols->length_row[j] = (int16_t) _mm_extract_epi16 (vHL, 7);
+            result->stats->rowcols->score_row[j] = (int8_t) _mm_extract_epi8 (vH, 15);
+            result->stats->rowcols->matches_row[j] = (int8_t) _mm_extract_epi8 (vHM, 15);
+            result->stats->rowcols->similar_row[j] = (int8_t) _mm_extract_epi8 (vHS, 15);
+            result->stats->rowcols->length_row[j] = (int8_t) _mm_extract_epi8 (vHL, 15);
 #endif
         }
     }
@@ -459,26 +475,26 @@ parasail_result_t* PNAME(
     if (s2_end)
     {
         for (k=0; k<position; ++k) {
-            vMaxH = _mm_slli_si128(vMaxH, 2);
-            vMaxM = _mm_slli_si128(vMaxM, 2);
-            vMaxS = _mm_slli_si128(vMaxS, 2);
-            vMaxL = _mm_slli_si128(vMaxL, 2);
+            vMaxH = _mm_slli_si128(vMaxH, 1);
+            vMaxM = _mm_slli_si128(vMaxM, 1);
+            vMaxS = _mm_slli_si128(vMaxS, 1);
+            vMaxL = _mm_slli_si128(vMaxL, 1);
         }
         end_query = s1Len-1;
-        score = (int16_t) _mm_extract_epi16(vMaxH, 7);
-        matches = (int16_t) _mm_extract_epi16(vMaxM, 7);
-        similar = (int16_t) _mm_extract_epi16(vMaxS, 7);
-        length = (int16_t) _mm_extract_epi16(vMaxL, 7);
+        score = (int8_t) _mm_extract_epi8(vMaxH, 15);
+        matches = (int8_t) _mm_extract_epi8(vMaxM, 15);
+        similar = (int8_t) _mm_extract_epi8(vMaxS, 15);
+        length = (int8_t) _mm_extract_epi8(vMaxL, 15);
     }
 
     /* max of last column */
     if (s1_end)
     {
         /* Trace the alignment ending position on read. */
-        int16_t *t = (int16_t*)pvH;
-        int16_t *m = (int16_t*)pvHM;
-        int16_t *s = (int16_t*)pvHS;
-        int16_t *l = (int16_t*)pvHL;
+        int8_t *t = (int8_t*)pvH;
+        int8_t *m = (int8_t*)pvHM;
+        int8_t *s = (int8_t*)pvHS;
+        int8_t *l = (int8_t*)pvHL;
         int32_t column_len = segLen * segWidth;
         for (i = 0; i<column_len; ++i, ++t, ++m, ++s, ++l) {
             int32_t temp = i / segWidth + i % segWidth * segLen;
@@ -508,22 +524,22 @@ parasail_result_t* PNAME(
         __m128i vHS = _mm_load_si128(pvHS + offset);
         __m128i vHL = _mm_load_si128(pvHL + offset);
         for (k=0; k<position; ++k) {
-            vH = _mm_slli_si128(vH, 2);
-            vHM = _mm_slli_si128(vHM, 2);
-            vHS = _mm_slli_si128(vHS, 2);
-            vHL = _mm_slli_si128(vHL, 2);
+            vH = _mm_slli_si128(vH, 1);
+            vHM = _mm_slli_si128(vHM, 1);
+            vHS = _mm_slli_si128(vHS, 1);
+            vHL = _mm_slli_si128(vHL, 1);
         }
         end_ref = s2Len - 1;
         end_query = s1Len - 1;
-        score = (int16_t) _mm_extract_epi16 (vH, 7);
-        matches = (int16_t) _mm_extract_epi16 (vHM, 7);
-        similar = (int16_t) _mm_extract_epi16 (vHS, 7);
-        length = (int16_t) _mm_extract_epi16 (vHL, 7);
+        score = (int8_t) _mm_extract_epi8 (vH, 15);
+        matches = (int8_t) _mm_extract_epi8 (vHM, 15);
+        similar = (int8_t) _mm_extract_epi8 (vHS, 15);
+        length = (int8_t) _mm_extract_epi8 (vHL, 15);
     }
 
     if (_mm_movemask_epi8(_mm_or_si128(
-            _mm_cmplt_epi16(vSaturationCheckMin, vNegLimit),
-            _mm_cmpgt_epi16(vSaturationCheckMax, vPosLimit)))) {
+            _mm_cmplt_epi8(vSaturationCheckMin, vNegLimit),
+            _mm_cmpgt_epi8(vSaturationCheckMax, vPosLimit)))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
         score = 0;
         matches = 0;
@@ -541,7 +557,7 @@ parasail_result_t* PNAME(
     result->stats->length = length;
     result->flag |= PARASAIL_FLAG_SG | PARASAIL_FLAG_SCAN
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_16 | PARASAIL_FLAG_LANES_8;
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
     result->flag |= s1_beg ? PARASAIL_FLAG_SG_S1_BEG : 0;
     result->flag |= s1_end ? PARASAIL_FLAG_SG_S1_END : 0;
     result->flag |= s2_beg ? PARASAIL_FLAG_SG_S2_BEG : 0;
