@@ -21,7 +21,7 @@
 #define SWAP(A,B) { simde__m128i* tmp = A; A = B; B = tmp; }
 #define SWAP3(A,B,C) { simde__m128i* tmp = A; A = B; B = C; C = tmp; }
 
-#define NEG_INF (INT64_MIN/(int64_t)(2))
+#define NEG_INF INT8_MIN
 
 
 #ifdef PARASAIL_TABLE
@@ -31,10 +31,25 @@ static inline void arr_store(
         int32_t t,
         int32_t seglen,
         int32_t d,
-        int32_t dlen)
+        int32_t dlen,
+        int32_t bias)
 {
-    array[1LL*(0*seglen+t)*dlen + d] = (int64_t)simde_mm_extract_epi64(vH, 0);
-    array[1LL*(1*seglen+t)*dlen + d] = (int64_t)simde_mm_extract_epi64(vH, 1);
+    array[1LL*( 0*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  0) - bias;
+    array[1LL*( 1*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  1) - bias;
+    array[1LL*( 2*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  2) - bias;
+    array[1LL*( 3*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  3) - bias;
+    array[1LL*( 4*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  4) - bias;
+    array[1LL*( 5*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  5) - bias;
+    array[1LL*( 6*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  6) - bias;
+    array[1LL*( 7*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  7) - bias;
+    array[1LL*( 8*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  8) - bias;
+    array[1LL*( 9*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH,  9) - bias;
+    array[1LL*(10*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 10) - bias;
+    array[1LL*(11*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 11) - bias;
+    array[1LL*(12*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 12) - bias;
+    array[1LL*(13*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 13) - bias;
+    array[1LL*(14*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 14) - bias;
+    array[1LL*(15*seglen+t)*dlen + d] = (int8_t)simde_mm_extract_epi8(vH, 15) - bias;
 }
 #endif
 
@@ -43,32 +58,47 @@ static inline void arr_store_col(
         int *col,
         simde__m128i vH,
         int32_t t,
-        int32_t seglen)
+        int32_t seglen,
+        int32_t bias)
 {
-    col[0*seglen+t] = (int64_t)simde_mm_extract_epi64(vH, 0);
-    col[1*seglen+t] = (int64_t)simde_mm_extract_epi64(vH, 1);
+    col[ 0*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  0) - bias;
+    col[ 1*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  1) - bias;
+    col[ 2*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  2) - bias;
+    col[ 3*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  3) - bias;
+    col[ 4*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  4) - bias;
+    col[ 5*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  5) - bias;
+    col[ 6*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  6) - bias;
+    col[ 7*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  7) - bias;
+    col[ 8*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  8) - bias;
+    col[ 9*seglen+t] = (int8_t)simde_mm_extract_epi8(vH,  9) - bias;
+    col[10*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 10) - bias;
+    col[11*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 11) - bias;
+    col[12*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 12) - bias;
+    col[13*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 13) - bias;
+    col[14*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 14) - bias;
+    col[15*seglen+t] = (int8_t)simde_mm_extract_epi8(vH, 15) - bias;
 }
 #endif
 
 #ifdef PARASAIL_TABLE
-#define FNAME parasail_sw_stats_table_striped_neon_128_64
-#define PNAME parasail_sw_stats_table_striped_profile_neon_128_64
+#define FNAME parasail_sw_stats_table_striped_neon_128_8
+#define PNAME parasail_sw_stats_table_striped_profile_neon_128_8
 #define INAME PNAME
 #define STATIC
 #else
 #ifdef PARASAIL_ROWCOL
-#define FNAME parasail_sw_stats_rowcol_striped_neon_128_64
-#define PNAME parasail_sw_stats_rowcol_striped_profile_neon_128_64
+#define FNAME parasail_sw_stats_rowcol_striped_neon_128_8
+#define PNAME parasail_sw_stats_rowcol_striped_profile_neon_128_8
 #define INAME PNAME
 #define STATIC
 #else
-#define FNAME parasail_sw_stats_striped_neon_128_64
+#define FNAME parasail_sw_stats_striped_neon_128_8
 #ifdef FASTSTATS
-#define PNAME parasail_sw_stats_striped_profile_neon_128_64_internal
-#define INAME parasail_sw_stats_striped_profile_neon_128_64
+#define PNAME parasail_sw_stats_striped_profile_neon_128_8_internal
+#define INAME parasail_sw_stats_striped_profile_neon_128_8
 #define STATIC static
 #else
-#define PNAME parasail_sw_stats_striped_profile_neon_128_64
+#define PNAME parasail_sw_stats_striped_profile_neon_128_8
 #define INAME PNAME
 #define STATIC
 #endif
@@ -80,7 +110,7 @@ parasail_result_t* FNAME(
         const char * const restrict s2, const int s2Len,
         const int open, const int gap, const parasail_matrix_t *matrix)
 {
-    parasail_profile_t *profile = parasail_profile_create_stats_neon_128_64(s1, s1Len, matrix);
+    parasail_profile_t *profile = parasail_profile_create_stats_neon_128_8(s1, s1Len, matrix);
     parasail_result_t *result = INAME(profile, s2, s2Len, open, gap);
     parasail_profile_free(profile);
     return result;
@@ -98,11 +128,11 @@ STATIC parasail_result_t* PNAME(
     int32_t end_ref = 0;
     const int s1Len = profile->s1Len;
     const parasail_matrix_t *matrix = profile->matrix;
-    const int32_t segWidth = 2; /* number of values in vector unit */
+    const int32_t segWidth = 16; /* number of values in vector unit */
     const int32_t segLen = (s1Len + segWidth - 1) / segWidth;
-    simde__m128i* const restrict vProfile  = (simde__m128i*)profile->profile64.score;
-    simde__m128i* const restrict vProfileM = (simde__m128i*)profile->profile64.matches;
-    simde__m128i* const restrict vProfileS = (simde__m128i*)profile->profile64.similar;
+    simde__m128i* const restrict vProfile  = (simde__m128i*)profile->profile8.score;
+    simde__m128i* const restrict vProfileM = (simde__m128i*)profile->profile8.matches;
+    simde__m128i* const restrict vProfileS = (simde__m128i*)profile->profile8.similar;
     simde__m128i* restrict pvHStore        = parasail_memalign_simde__m128i(16, segLen);
     simde__m128i* restrict pvHLoad         = parasail_memalign_simde__m128i(16, segLen);
     simde__m128i* restrict pvHMStore       = parasail_memalign_simde__m128i(16, segLen);
@@ -116,22 +146,28 @@ STATIC parasail_result_t* PNAME(
     simde__m128i* const restrict pvES      = parasail_memalign_simde__m128i(16, segLen);
     simde__m128i* const restrict pvEL      = parasail_memalign_simde__m128i(16, segLen);
     simde__m128i* restrict pvHMax          = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* restrict pvHMMax          = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* restrict pvHSMax          = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i* restrict pvHLMax          = parasail_memalign_simde__m128i(16, segLen);
-    simde__m128i vGapO = simde_mm_set1_epi64x(open);
-    simde__m128i vGapE = simde_mm_set1_epi64x(gap);
+    simde__m128i* restrict pvHMMax         = parasail_memalign_simde__m128i(16, segLen);
+    simde__m128i* restrict pvHSMax         = parasail_memalign_simde__m128i(16, segLen);
+    simde__m128i* restrict pvHLMax         = parasail_memalign_simde__m128i(16, segLen);
+    simde__m128i vGapO = simde_mm_set1_epi8(open);
+    simde__m128i vGapE = simde_mm_set1_epi8(gap);
     simde__m128i vZero = simde_mm_setzero_si128();
-    simde__m128i vOne = simde_mm_set1_epi64x(1);
-    int64_t score = NEG_INF;
-    int64_t matches = NEG_INF;
-    int64_t similar = NEG_INF;
-    int64_t length = NEG_INF;
-    simde__m128i vMaxH = vZero;
-    simde__m128i vMaxHUnit = vZero;
-    simde__m128i vSaturationCheckMax = vZero;
-    simde__m128i vPosLimit = simde_mm_set1_epi64x(INT64_MAX);
-    int64_t maxp = INT64_MAX - (int64_t)(matrix->max+1);
+    simde__m128i vOne = simde_mm_set1_epi8(1);
+    int8_t bias = INT8_MIN;
+    int8_t score = bias;
+    int8_t matches = bias;
+    int8_t similar = bias;
+    int8_t length = bias;
+    simde__m128i vBias = simde_mm_set1_epi8(bias);
+    simde__m128i vBias1 = simde_mm_adds_epi8(vBias,vOne);
+    simde__m128i vMaxH = vBias;
+    simde__m128i vMaxHUnit = vBias;
+    simde__m128i insert_mask = simde_mm_cmpgt_epi8(
+            simde_mm_set_epi8(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1),
+            vZero);
+    simde__m128i vSaturationCheckMax = vBias;
+    simde__m128i vPosLimit = simde_mm_set1_epi8(INT8_MAX);
+    int8_t maxp = INT8_MAX - (int8_t)(matrix->max+1);
 #ifdef PARASAIL_TABLE
     parasail_result_t *result = parasail_result_new_table3(segLen*segWidth, s2Len);
 #else
@@ -144,14 +180,14 @@ STATIC parasail_result_t* PNAME(
 #endif
 #endif
 
-    parasail_memset_simde__m128i(pvHStore, vZero, segLen);
-    parasail_memset_simde__m128i(pvHMStore, vZero, segLen);
-    parasail_memset_simde__m128i(pvHSStore, vZero, segLen);
-    parasail_memset_simde__m128i(pvHLStore, vZero, segLen);
-    parasail_memset_simde__m128i(pvE, simde_mm_set1_epi64x(-open), segLen);
-    parasail_memset_simde__m128i(pvEM, vZero, segLen);
-    parasail_memset_simde__m128i(pvES, vZero, segLen);
-    parasail_memset_simde__m128i(pvEL, vOne, segLen);
+    parasail_memset_simde__m128i(pvHStore, vBias, segLen);
+    parasail_memset_simde__m128i(pvHMStore, vBias, segLen);
+    parasail_memset_simde__m128i(pvHSStore, vBias, segLen);
+    parasail_memset_simde__m128i(pvHLStore, vBias, segLen);
+    parasail_memset_simde__m128i(pvE, vBias, segLen);
+    parasail_memset_simde__m128i(pvEM, vBias, segLen);
+    parasail_memset_simde__m128i(pvES, vBias, segLen);
+    parasail_memset_simde__m128i(pvEL, vBias1, segLen);
 
     /* outer loop over database sequence */
     for (j=0; j<s2Len; ++j) {
@@ -177,20 +213,24 @@ STATIC parasail_result_t* PNAME(
 
         /* Initialize F value to 0.  Any errors to vH values will be
          * corrected in the Lazy_F loop. */
-        vF = vZero;
-        vFM = vZero;
-        vFS = vZero;
-        vFL = vOne;
+        vF = vBias;
+        vFM = vBias;
+        vFS = vBias;
+        vFL = vBias1;
 
-        /* load final segment of pvHStore and shift left by 8 bytes */
+        /* load final segment of pvHStore and shift left by 1 bytes */
         vH = simde_mm_load_si128(&pvHStore[segLen - 1]);
         vHM = simde_mm_load_si128(&pvHMStore[segLen - 1]);
         vHS = simde_mm_load_si128(&pvHSStore[segLen - 1]);
         vHL = simde_mm_load_si128(&pvHLStore[segLen - 1]);
-        vH = simde_mm_slli_si128(vH, 8);
-        vHM = simde_mm_slli_si128(vHM, 8);
-        vHS = simde_mm_slli_si128(vHS, 8);
-        vHL = simde_mm_slli_si128(vHL, 8);
+        vH = simde_mm_slli_si128(vH, 1);
+        vHM = simde_mm_slli_si128(vHM, 1);
+        vHS = simde_mm_slli_si128(vHS, 1);
+        vHL = simde_mm_slli_si128(vHL, 1);
+        vH = simde_mm_blendv_epi8(vH, vBias, insert_mask);
+        vHM = simde_mm_blendv_epi8(vHM, vBias, insert_mask);
+        vHS = simde_mm_blendv_epi8(vHS, vBias, insert_mask);
+        vHL = simde_mm_blendv_epi8(vHL, vBias, insert_mask);
 
         /* Correct part of the vProfile */
         vP = vProfile + matrix->mapper[(unsigned char)s2[j]] * segLen;
@@ -224,59 +264,59 @@ STATIC parasail_result_t* PNAME(
             vEL = simde_mm_load_si128(pvEL+ i);
 
             /* Get max from vH, vE and vF. */
-            vH_dag = simde_mm_add_epi64(vH, simde_mm_load_si128(vP + i));
-            vH_dag = simde_mm_max_epi64(vH_dag, vZero);
-            vH = simde_mm_max_epi64(vH_dag, vE);
-            vH = simde_mm_max_epi64(vH, vF);
+            vH_dag = simde_mm_adds_epi8(vH, simde_mm_load_si128(vP + i));
+            /*vH = simde_mm_max_epi8(vH_dag, vBias);*/
+            vH = simde_mm_max_epi8(vH_dag, vE);
+            vH = simde_mm_max_epi8(vH, vF);
             /* Save vH values. */
             simde_mm_store_si128(pvHStore + i, vH);
-            cond_zero = simde_mm_cmpeq_epi64(vH, vZero);
+            cond_zero = simde_mm_cmpeq_epi8(vH, vBias);
 
-            case1 = simde_mm_cmpeq_epi64(vH, vH_dag);
-            case2 = simde_mm_cmpeq_epi64(vH, vF);
+            case1 = simde_mm_cmpeq_epi8(vH, vH_dag);
+            case2 = simde_mm_cmpeq_epi8(vH, vF);
 
             /* calculate vM */
             vHM = simde_mm_blendv_epi8(
                     simde_mm_blendv_epi8(vEM, vFM, case2),
-                    simde_mm_add_epi64(vHM, simde_mm_load_si128(vPM + i)), case1);
-            vHM = simde_mm_andnot_si128(cond_zero, vHM);
+                    simde_mm_adds_epi8(vHM, simde_mm_load_si128(vPM + i)), case1);
+            vHM = simde_mm_blendv_epi8(vHM, vBias, cond_zero);
             simde_mm_store_si128(pvHMStore + i, vHM);
 
             /* calculate vS */
             vHS = simde_mm_blendv_epi8(
                     simde_mm_blendv_epi8(vES, vFS, case2),
-                    simde_mm_add_epi64(vHS, simde_mm_load_si128(vPS + i)), case1);
-            vHS = simde_mm_andnot_si128(cond_zero, vHS);
+                    simde_mm_adds_epi8(vHS, simde_mm_load_si128(vPS + i)), case1);
+            vHS = simde_mm_blendv_epi8(vHS, vBias, cond_zero);
             simde_mm_store_si128(pvHSStore + i, vHS);
 
             /* calculate vL */
             vHL = simde_mm_blendv_epi8(
                     simde_mm_blendv_epi8(vEL, vFL, case2),
-                    simde_mm_add_epi64(vHL, vOne), case1);
-            vHL = simde_mm_andnot_si128(cond_zero, vHL);
+                    simde_mm_adds_epi8(vHL, vOne), case1);
+            vHL = simde_mm_blendv_epi8(vHL, vBias, cond_zero);
             simde_mm_store_si128(pvHLStore + i, vHL);
 
-            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHM);
-            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHS);
-            vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHL);
+            vSaturationCheckMax = simde_mm_max_epi8(vSaturationCheckMax, vHM);
+            vSaturationCheckMax = simde_mm_max_epi8(vSaturationCheckMax, vHS);
+            vSaturationCheckMax = simde_mm_max_epi8(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
-            arr_store(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
-            arr_store(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
-            arr_store(result->stats->tables->length_table, vHL, i, segLen, j, s2Len);
-            arr_store(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
+            arr_store(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len, bias);
+            arr_store(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len, bias);
+            arr_store(result->stats->tables->length_table, vHL, i, segLen, j, s2Len, bias);
+            arr_store(result->stats->tables->score_table, vH, i, segLen, j, s2Len, bias);
 #endif
-            vMaxH = simde_mm_max_epi64(vH, vMaxH);
-            vEF_opn = simde_mm_sub_epi64(vH, vGapO);
+            vMaxH = simde_mm_max_epi8(vH, vMaxH);
+            vEF_opn = simde_mm_subs_epi8(vH, vGapO);
 
             /* Update vE value. */
-            vE_ext = simde_mm_sub_epi64(vE, vGapE);
-            vE = simde_mm_max_epi64(vEF_opn, vE_ext);
-            case1 = simde_mm_cmpgt_epi64(vEF_opn, vE_ext);
+            vE_ext = simde_mm_subs_epi8(vE, vGapE);
+            vE = simde_mm_max_epi8(vEF_opn, vE_ext);
+            case1 = simde_mm_cmpgt_epi8(vEF_opn, vE_ext);
             vEM = simde_mm_blendv_epi8(vEM, vHM, case1);
             vES = simde_mm_blendv_epi8(vES, vHS, case1);
             vEL = simde_mm_blendv_epi8(
-                    simde_mm_add_epi64(vEL, vOne),
-                    simde_mm_add_epi64(vHL, vOne),
+                    simde_mm_adds_epi8(vEL, vOne),
+                    simde_mm_adds_epi8(vHL, vOne),
                     case1);
             simde_mm_store_si128(pvE + i, vE);
             simde_mm_store_si128(pvEM + i, vEM);
@@ -284,14 +324,14 @@ STATIC parasail_result_t* PNAME(
             simde_mm_store_si128(pvEL + i, vEL);
 
             /* Update vF value. */
-            vF_ext = simde_mm_sub_epi64(vF, vGapE);
-            vF = simde_mm_max_epi64(vEF_opn, vF_ext);
-            case1 = simde_mm_cmpgt_epi64(vEF_opn, vF_ext);
+            vF_ext = simde_mm_subs_epi8(vF, vGapE);
+            vF = simde_mm_max_epi8(vEF_opn, vF_ext);
+            case1 = simde_mm_cmpgt_epi8(vEF_opn, vF_ext);
             vFM = simde_mm_blendv_epi8(vFM, vHM, case1);
             vFS = simde_mm_blendv_epi8(vFS, vHS, case1);
             vFL = simde_mm_blendv_epi8(
-                    simde_mm_add_epi64(vFL, vOne),
-                    simde_mm_add_epi64(vHL, vOne),
+                    simde_mm_adds_epi8(vFL, vOne),
+                    simde_mm_adds_epi8(vHL, vOne),
                     case1);
 
             /* Load the next vH. */
@@ -305,25 +345,27 @@ STATIC parasail_result_t* PNAME(
          * then deletion, so don't update E(i, i), learn from SWPS3 */
         for (k=0; k<segWidth; ++k) {
             simde__m128i vHp = simde_mm_load_si128(&pvHLoad[segLen - 1]);
-            vHp = simde_mm_slli_si128(vHp, 8);
-            vF = simde_mm_slli_si128(vF, 8);
-            vF = simde_mm_insert_epi64(vF, -open, 0);
-            vFM = simde_mm_slli_si128(vFM, 8);
-            vFS = simde_mm_slli_si128(vFS, 8);
-            vFL = simde_mm_slli_si128(vFL, 8);
-            vFL = simde_mm_insert_epi64(vFL, 1, 0);
+            vHp = simde_mm_slli_si128(vHp, 1);
+            vF = simde_mm_slli_si128(vF, 1);
+            vFM = simde_mm_slli_si128(vFM, 1);
+            vFS = simde_mm_slli_si128(vFS, 1);
+            vFL = simde_mm_slli_si128(vFL, 1);
+            vHp = simde_mm_blendv_epi8(vHp, vBias, insert_mask);
+            vF = simde_mm_blendv_epi8(vF, vBias, insert_mask);
+            vFM = simde_mm_blendv_epi8(vFM, vBias, insert_mask);
+            vFS = simde_mm_blendv_epi8(vFS, vBias, insert_mask);
+            vFL = simde_mm_blendv_epi8(vFL, vBias1, insert_mask);
             for (i=0; i<segLen; ++i) {
                 simde__m128i case1;
                 simde__m128i case2;
                 simde__m128i cond;
 
-                vHp = simde_mm_add_epi64(vHp, simde_mm_load_si128(vP + i));
-                vHp = simde_mm_max_epi64(vHp, vZero);
+                vHp = simde_mm_adds_epi8(vHp, simde_mm_load_si128(vP + i));
                 vH = simde_mm_load_si128(pvHStore + i);
-                vH = simde_mm_max_epi64(vH,vF);
+                vH = simde_mm_max_epi8(vH, vF);
                 simde_mm_store_si128(pvHStore + i, vH);
-                case1 = simde_mm_cmpeq_epi64(vH, vHp);
-                case2 = simde_mm_cmpeq_epi64(vH, vF);
+                case1 = simde_mm_cmpeq_epi8(vH, vHp);
+                case2 = simde_mm_cmpeq_epi8(vH, vF);
                 cond = simde_mm_andnot_si128(case1, case2);
 
                 /* calculate vM */
@@ -341,34 +383,34 @@ STATIC parasail_result_t* PNAME(
                 vHL = simde_mm_blendv_epi8(vHL, vFL, cond);
                 simde_mm_store_si128(pvHLStore + i, vHL);
 
-                vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHM);
-                vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHS);
-                vSaturationCheckMax = simde_mm_max_epi64(vSaturationCheckMax, vHL);
+                vSaturationCheckMax = simde_mm_max_epi8(vSaturationCheckMax, vHM);
+                vSaturationCheckMax = simde_mm_max_epi8(vSaturationCheckMax, vHS);
+                vSaturationCheckMax = simde_mm_max_epi8(vSaturationCheckMax, vHL);
 #ifdef PARASAIL_TABLE
-                arr_store(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len);
-                arr_store(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len);
-                arr_store(result->stats->tables->length_table, vHL, i, segLen, j, s2Len);
-                arr_store(result->stats->tables->score_table, vH, i, segLen, j, s2Len);
+                arr_store(result->stats->tables->matches_table, vHM, i, segLen, j, s2Len, bias);
+                arr_store(result->stats->tables->similar_table, vHS, i, segLen, j, s2Len, bias);
+                arr_store(result->stats->tables->length_table, vHL, i, segLen, j, s2Len, bias);
+                arr_store(result->stats->tables->score_table, vH, i, segLen, j, s2Len, bias);
 #endif
-                vMaxH = simde_mm_max_epi64(vH, vMaxH);
+                vMaxH = simde_mm_max_epi8(vH, vMaxH);
                 /* Update vF value. */
-                vEF_opn = simde_mm_sub_epi64(vH, vGapO);
-                vF_ext = simde_mm_sub_epi64(vF, vGapE);
+                vEF_opn = simde_mm_subs_epi8(vH, vGapO);
+                vF_ext = simde_mm_subs_epi8(vF, vGapE);
                 if (! simde_mm_movemask_epi8(
                             simde_mm_or_si128(
-                                simde_mm_cmpgt_epi64(vF_ext, vEF_opn),
+                                simde_mm_cmpgt_epi8(vF_ext, vEF_opn),
                                 simde_mm_and_si128(
-                                    simde_mm_cmpeq_epi64(vF_ext, vEF_opn),
-                                    simde_mm_cmpgt_epi64(vF_ext, vZero)))))
+                                    simde_mm_cmpeq_epi8(vF_ext, vEF_opn),
+                                    simde_mm_cmpgt_epi8(vF_ext, vBias)))))
                     goto end;
-                /*vF = simde_mm_max_epi64(vEF_opn, vF_ext);*/
+                /*vF = simde_mm_max_epi8(vEF_opn, vF_ext);*/
                 vF = vF_ext;
-                cond = simde_mm_cmpgt_epi64(vEF_opn, vF_ext);
+                cond = simde_mm_cmpgt_epi8(vEF_opn, vF_ext);
                 vFM = simde_mm_blendv_epi8(vFM, vHM, cond);
                 vFS = simde_mm_blendv_epi8(vFS, vHS, cond);
                 vFL = simde_mm_blendv_epi8(
-                        simde_mm_add_epi64(vFL, vOne),
-                        simde_mm_add_epi64(vHL, vOne),
+                        simde_mm_adds_epi8(vFL, vOne),
+                        simde_mm_adds_epi8(vHL, vOne),
                         cond);
                 vHp = simde_mm_load_si128(pvHLoad + i);
             }
@@ -385,28 +427,28 @@ end:
             vHS = simde_mm_load_si128(pvHSStore + offset);
             vHL = simde_mm_load_si128(pvHLStore + offset);
             for (k=0; k<position; ++k) {
-                vH = simde_mm_slli_si128(vH, 8);
-                vHM = simde_mm_slli_si128(vHM, 8);
-                vHS = simde_mm_slli_si128(vHS, 8);
-                vHL = simde_mm_slli_si128(vHL, 8);
+                vH = simde_mm_slli_si128(vH, 1);
+                vHM = simde_mm_slli_si128(vHM, 1);
+                vHS = simde_mm_slli_si128(vHS, 1);
+                vHL = simde_mm_slli_si128(vHL, 1);
             }
-            result->stats->rowcols->score_row[j] = (int64_t) simde_mm_extract_epi64 (vH, 1);
-            result->stats->rowcols->matches_row[j] = (int64_t) simde_mm_extract_epi64 (vHM, 1);
-            result->stats->rowcols->similar_row[j] = (int64_t) simde_mm_extract_epi64 (vHS, 1);
-            result->stats->rowcols->length_row[j] = (int64_t) simde_mm_extract_epi64 (vHL, 1);
+            result->stats->rowcols->score_row[j] = (int8_t) simde_mm_extract_epi8 (vH, 15) - bias;
+            result->stats->rowcols->matches_row[j] = (int8_t) simde_mm_extract_epi8 (vHM, 15) - bias;
+            result->stats->rowcols->similar_row[j] = (int8_t) simde_mm_extract_epi8 (vHS, 15) - bias;
+            result->stats->rowcols->length_row[j] = (int8_t) simde_mm_extract_epi8 (vHL, 15) - bias;
         }
 #endif
 
         {
-            simde__m128i vCompare = simde_mm_cmpgt_epi64(vMaxH, vMaxHUnit);
+            simde__m128i vCompare = simde_mm_cmpgt_epi8(vMaxH, vMaxHUnit);
             if (simde_mm_movemask_epi8(vCompare)) {
-                score = simde_mm_hmax_epi64(vMaxH);
+                score = simde_mm_hmax_epi8(vMaxH);
                 /* if score has potential to overflow, abort early */
                 if (score > maxp) {
                     result->flag |= PARASAIL_FLAG_SATURATED;
                     break;
                 }
-                vMaxHUnit = simde_mm_set1_epi64x(score);
+                vMaxHUnit = simde_mm_set1_epi8(score);
                 end_ref = j;
             }
         }
@@ -418,15 +460,15 @@ end:
         simde__m128i vHM = simde_mm_load_si128(pvHMStore+i);
         simde__m128i vHS = simde_mm_load_si128(pvHSStore+i);
         simde__m128i vHL = simde_mm_load_si128(pvHLStore+i);
-        arr_store_col(result->stats->rowcols->score_col, vH, i, segLen);
-        arr_store_col(result->stats->rowcols->matches_col, vHM, i, segLen);
-        arr_store_col(result->stats->rowcols->similar_col, vHS, i, segLen);
-        arr_store_col(result->stats->rowcols->length_col, vHL, i, segLen);
+        arr_store_col(result->stats->rowcols->score_col, vH, i, segLen, bias);
+        arr_store_col(result->stats->rowcols->matches_col, vHM, i, segLen, bias);
+        arr_store_col(result->stats->rowcols->similar_col, vHS, i, segLen, bias);
+        arr_store_col(result->stats->rowcols->length_col, vHL, i, segLen, bias);
     }
 #endif
 
-    if (score == INT64_MAX
-            || simde_mm_movemask_epi8(simde_mm_cmpeq_epi64(vSaturationCheckMax,vPosLimit))) {
+    if (score == INT8_MAX
+            || simde_mm_movemask_epi8(simde_mm_cmpeq_epi8(vSaturationCheckMax,vPosLimit))) {
         result->flag |= PARASAIL_FLAG_SATURATED;
     }
 
@@ -455,10 +497,10 @@ end:
         }
         /* Trace the alignment ending position on read. */
         {
-            int64_t *t = (int64_t*)pvHMax;
-            int64_t *m = (int64_t*)pvHMMax;
-            int64_t *s = (int64_t*)pvHSMax;
-            int64_t *l = (int64_t*)pvHLMax;
+            int8_t *t = (int8_t*)pvHMax;
+            int8_t *m = (int8_t*)pvHMMax;
+            int8_t *s = (int8_t*)pvHSMax;
+            int8_t *l = (int8_t*)pvHLMax;
             int32_t column_len = segLen * segWidth;
             end_query = s1Len;
             for (i = 0; i<column_len; ++i, ++t, ++m, ++s, ++l) {
@@ -475,15 +517,15 @@ end:
         }
     }
 
-    result->score = score;
+    result->score = score - bias;
     result->end_query = end_query;
     result->end_ref = end_ref;
-    result->stats->matches = matches;
-    result->stats->similar = similar;
-    result->stats->length = length;
+    result->stats->matches = matches - bias;
+    result->stats->similar = similar - bias;
+    result->stats->length = length - bias;
     result->flag |= PARASAIL_FLAG_SW | PARASAIL_FLAG_STRIPED
         | PARASAIL_FLAG_STATS
-        | PARASAIL_FLAG_BITS_64 | PARASAIL_FLAG_LANES_2;
+        | PARASAIL_FLAG_BITS_8 | PARASAIL_FLAG_LANES_16;
 #ifdef PARASAIL_TABLE
     result->flag |= PARASAIL_FLAG_TABLE;
 #endif
@@ -526,7 +568,7 @@ parasail_result_t* INAME(
     const parasail_matrix_t *matrix = profile->matrix;
 
     /* find the end loc first with the faster implementation */
-    parasail_result_t *result = parasail_sw_striped_profile_neon_128_64(profile, s2, s2Len, open, gap);
+    parasail_result_t *result = parasail_sw_striped_profile_neon_128_8(profile, s2, s2Len, open, gap);
     if (!parasail_result_is_saturated(result)) {
 #if 0
         int s1Len_new = 0;
@@ -548,10 +590,10 @@ parasail_result_t* INAME(
         s2Len_new = result->end_ref+1;
         s1_new = parasail_reverse(s1, s1Len_new);
         s2_new = parasail_reverse(s2, s2Len_new);
-        profile_new = parasail_profile_create_neon_128_64(
+        profile_new = parasail_profile_create_neon_128_8(
                 s1_new, s1Len_new, matrix);
         profile_new->stop = result->score;
-        result_new = parasail_sw_striped_profile_neon_128_64(
+        result_new = parasail_sw_striped_profile_neon_128_8(
                 profile_new, s2_new, s2Len_new, open, gap);
 
         /* using both the beg and end loc, call the original stats func */
@@ -563,7 +605,7 @@ parasail_result_t* INAME(
         assert(s2_begin >= 0);
         assert(s1Len_new > s1_begin);
         assert(s2Len_new > s2_begin);
-        profile_final = parasail_profile_create_stats_neon_128_64(
+        profile_final = parasail_profile_create_stats_neon_128_8(
                 &s1[s1_begin], s1Len_final, matrix);
         result_final = PNAME(
                 profile_final, &s2[s2_begin], s2Len_final, open, gap);
@@ -589,7 +631,7 @@ parasail_result_t* INAME(
         /* using the end loc, call the original stats function */
         s1Len_new = result->end_query+1;
         s2Len_new = result->end_ref+1;
-        profile_final = parasail_profile_create_stats_neon_128_64(
+        profile_final = parasail_profile_create_stats_neon_128_8(
                 s1, s1Len_new, matrix);
         result_final = PNAME(
                 profile_final, s2, s2Len_new, open, gap);
